import 'package:json_annotation/json_annotation.dart';

part 'generic_document.g.dart';

/// Barcode document format
enum ValidationStatus {
  /// Field value failed validation */
  Invalid,

  /// Field value passed validation */
  Valid,
}

/// Common field types
enum CommonFieldType {
  /// Default value */
  None,

  /// Document ID */
  ID,

  /// Person surname name field. */
  Surname,

  /// Person maiden name field. */
  MaidenName,

  /// Person given names field */
  GivenNames,

  /// Person birth date field */
  BirthDate,

  /// Person nationality field */
  Nationality,

  /// Person birthplace field */
  Birthplace,

  /// Document expiry date field */
  ExpiryDate,

  /// Person eye color field */
  EyeColor,

  /// Person height field */
  Height,

  /// Document issue date field */
  IssueDate,

  /// Document issuing authority field */
  IssuingAuthority,

  /// Address field */
  Address,

  /// Pseudonym field */
  Pseudonym,

  /// MRZ field */
  MRZ,

  /// Country code (ISO-3166 Alpha-3) field */
  CountryCode,

  /// Gender field */
  Gender,

  /// Signature field */
  Signature,

  /// Photo field */
  Photo,

  /// Date of start of validity field */
  ValidFromDate,

  /// Check routing number */
  RoutingNumber,

  /// Check account number */
  AccountNumber,
}

/// Result of OCR text recognition.
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: false,
    explicitToJson: false)
class OcrResult {
  /// Recognized text
  String? text;

  /// Confidence in result accuracy. The value ranges from 0 to 100, higher is better.
  double? confidence;

  OcrResult({
    required this.text,
    required this.confidence,
  });

  factory OcrResult.fromJson(Map<String, dynamic> json) =>
      _$OcrResultFromJson(json);
}

/// Field type
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: false,
    explicitToJson: false)
class FieldType {
  /// Local field type name scoped to the containing document type.
  String name;

  /// Unique global field type name prefixed with the document types of all containing documents.
  String fullName;

  /// Normalized global field type name. Fields in document types derived from the same base document type in the schema will have the same normalized name.
  String normalizedName;

  /// optional common field type. Commonly occurring fields that have the same semantic meaning in different document types will often have a set common type.
  CommonFieldType? commonType;

  FieldType({
    required this.name,
    required this.fullName,
    required this.normalizedName,
    this.commonType,
  });

  factory FieldType.fromJson(Map<String, dynamic> json) =>
      _$FieldTypeFromJson(json);
}

/// Field type
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: false,
    explicitToJson: false)
class GenericDocumentNormalizedType {
  /// Local document type name.
  String? name;

  /// Unique global document type name prefixed with the document types of all containing documents.
  String? fullName;

  /// Normalized global document type name. Common document types appearing as child documents in different places will often have the same normalized type name.
  String? normalizedName;

  GenericDocumentNormalizedType({
    required this.name,
    required this.fullName,
    required this.normalizedName,
  });

  factory GenericDocumentNormalizedType.fromJson(Map<String, dynamic> json) =>
      _$GenericDocumentNormalizedTypeFromJson(json);
}

/// Document field
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: false,
    explicitToJson: false)
class Field {
  /// The type of the field.
  FieldType type;

  /// Optional value of the field. Applicable only to text fields.
  OcrResult? value;

  /// Field validation status. Applicable only to fields that support some kind of validation.
  ValidationStatus? validationStatus;

  Field({
    required this.type,
    this.value,
    this.validationStatus,
  });

  factory Field.fromJson(Map<String, dynamic> json) => _$FieldFromJson(json);
}

/// A generic document. Contains fields and sub-documents.
@JsonSerializable(
    includeIfNull: false,
    createFactory: true,
    createToJson: false,
    explicitToJson: false)
class GenericDocument {
  /// The type of the document.
  GenericDocumentNormalizedType? type;

  /// A list of document fields.
  List<Field>? fields;

  /// A list of sub-documents.
  List<GenericDocument>? children;

  /// Confidence in result accuracy. The value ranges from 0 to 1, higher is better.
  double? confidence;

  /// The weight of the confidence. Can be used to calculate the weighted average confidence of two documents.
  double? confidenceWeight;

  GenericDocument({
    required this.type,
    this.fields,
    this.children,
    required this.confidence,
    required this.confidenceWeight,
  });

  factory GenericDocument.fromJson(Map<String, dynamic> json) =>
      _$GenericDocumentFromJson(json);
}
